#!/bin/sh -e
# Copyright 2019 Nikita Ermakov <arei@altlinux.org>
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

DNAME="$(mktemp -d ${TMP:-/tmp}/altocation-XXXX)"

cleanup()
{
    rm -rf "$DNAME"
}

help()
{
    cat << EOF
${0##*/} -- Generate vacation application.

Usage: $0 [options]

Options:
  -h | --help              print this help message
  -n | --name              full name (from)
  -p | --position          your position (from)
  -s | --since             since date
  -t | --till              till date
  -z | --sign              your signature (basically it is your full name)
  -o | --ofile             output file name
  -c | --config            path to the config file (default: ~/.altocation)

Configuration file example:
cat ~/.altocation
NAME="Your full name"
POSITION="Your position"
EOF
    exit 0
}

manual_input()
{
    cat << EOF
You could use config and cmdline options.
Please check the --help
EOF
    echo -ne "Full name (from) = "
    read NAME

    echo -ne "Position (from) = "
    read POSITION

    echo -ne "Since = "
    read SINCE

    echo -ne "Till = "
    read TILL

    echo -ne "Signature = "
    read SIGN

    echo -ne "Output file name = "
    read FNAME
}

cmdline_parse()
{
    # Parse command line arguments
    TMPARGS="$(getopt -n "$0" -o h,n:,p:,s:,t:,z:,o:,c: -l help,name:,position:,since:,till:,sign:,ofile:,config: -- "$@")" ||
	      help
    eval set -- "$TMPARGS"

    NS=
    while :
    do
        case "$1" in
            --)
                shift; break ;;
            -h|--help)
                help ;;
            -n|--name)
                shift; NAME="$1" ;;
            -p|--position)
                shift; POSITION="$1" ;;
            -s|--since)
                shift; SINCE="$1" ;;
            -t|--till)
                shift; TILL="$1" ;;
            -z|--sign) # zign!
                shift; SIGN="$1" ;;
            -o|--ofile)
                shift; FNAME="$1" ;;
            -c|--config)
                shift; CONFIG="$1" ;;
            *)
                help ;;
        esac
        shift
    done
}

trap cleanup EXIT HUP INT QUIT TERM

# Logo
cat << EOF | base64 -d -
ICAgICAgICAgICAgICAgICBfXy4uLS0tLS0nKQogICAgICAgLC4tLS5fIC4tJ18uLi0tLi4uLScK
ICAgICAgJy0iJy4gXy9fIC8gIC4uLS0nJyIiJy0uCiAgICAgIF8uLS0iIi4uLjouXzooXyAuLjoi
OjouIFwKICAgLi0nIC4uOjotLSIiXygjIykjKSInOi4gXCBcKSAgICBcIF98XyAvCiAgL186LTon
LyAgOl9fKCMjKSMjKSAgICApOiApICAgJy0uLycgICAnXC4tJwogICIgIC8gfCAgOicgOi8iIlwv
Ly8pICAvOi4nICAgIC0tKCAgICAgICApLS0KICAgIC8gOiggOiggOiggICAoIy8vKSAgIiAgICAg
ICAuLSdcLl9fXy4vJy0uCiAgIC8gOi98XCA6XF86XCAgIFwjLy9cICAgICAgICAgICAgLyAgfCAg
XAogICB8Oi8gfCAiIi0tJzpcICAgKCMvLykgICAgICAgICAgICAgICcKICAgXC8gIFwgOnwgIFwg
OlwgICgjLy8pCiAgICAgICAgXDpcICAgJy4nOi4gXCMvL1wKICAgICAgICAgJzp8ICAgICItLSco
Iy8vLykKICAgICAgICAgICAgICAgICAgICAoIy8vLykKICAgICAgICAgICAgICAgICAgICAoIy8v
LykKICAgICAgICAgICAgICAgICAgICAgXCMvLy9cCiAgICAgICAgICAgICAgICAgICAgICgjIy8v
LykKICAgICAgICAgICAgICAgICAgICAgKCMjLy8vKQogICAgICAgICAgICAgICAgICAgICAoIyMv
Ly8pCiAgICAgICAgICAgICAgICAgICAgICgjIy8vLykKICAgICAgICAgICAgICAgICAgICAgIFwj
Iy8vL1wKICAgICAgICAgICAgICAgICAgICAgICgjIyMvLy8pCiAgICAgICAgICAgICAgICAgICAg
ICAoc2p3Ly8vLylfXy4uLi0tLS0tLi4uLl9fCiAgICAgICAgICAgICAgICAgICAgICAoIy86Oicn
JyAgICAgICAgICAgICAgICAgIiItLS4uXwogICAgICAgICAgICAgICAgIF9fLi4tJycnICAgICAg
ICAgICAgICAgICAgICAgICAgICAgICAiLS5fCiAgICAgICAgIF9fLi4tLSIiICAgICAgICAgICAg
ICAgICAgICAgICAgICAgICAgICAgICAgICAgICAnLl8KX19fLi4tLSIiICAgICAgICAgICAgICAg
ICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICItLi5fX19fCiAgKF8gIiItLS0u
Li4uX19fICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgIF9fLi4uLS0iIiBfKQog
ICAgIiIiLS0uLi4gIF9fXyIiIiIiLS0tLS0uLi4uLi4uX19fX19fXy4uLi4uLi0tLS0iIiIgICAg
IC0tIiIiCiAgICAgICAgICAgICAgICAgICIiIiIgICAgICAgLS0tLi4uLi4gICBfX18uLi4uLS0t
LQo=
EOF
echo "Rule 42: Do not forget your towel!"

# Check config
[ -n "${CONFIG-}" ] || CONFIG="$HOME/.altocation"
if [ -f "$CONFIG" ]; then
    echo "Using $CONFIG"
    . "$CONFIG"
else
    CONFIG=
fi

if [ $# -ge 1 ]; then
    cmdline_parse "$@"
else
    [ -n "${CONFIG-}" ] || manual_input
fi

# Check variables
[ -n "$NAME" ]     || NAME='$K={M|P(M)}$'
[ -n "$POSITION" ] || POSITION="Unknown"
[ -n "$SINCE" ]    || SINCE="1 января 1970"
[ -n "$TILL" ]     || TILL="19 января 2038"
[ -n "SIGN" ]      || SIGN=""
[ -n "$FNAME" ]    || FNAME="vacation"

# generate TeX document
pushd "$DNAME" >/dev/null
cat > "vacation.tex" << EOF
\documentclass[a4paper,14pt]{extarticle}
\usepackage[T1,T2A]{fontenc}
\usepackage[utf8]{inputenc}
\usepackage[english,russian]{babel}
\usepackage[onehalfspacing]{setspace}
\usepackage{geometry}
\usepackage{enumitem}

\parindent=1cm
\geometry{left=3.0cm}
\geometry{right=1.0cm}
\geometry{top=2.5cm}
\geometry{bottom=2.5cm}
\setlist{nosep}
\thispagestyle{empty}

\begin{document}

\begin{itemize}[leftmargin=10cm]
\item[] Генеральному директору
\item[] ООО «Базальт СПО»
\item[] Смирнову А.В.
\item[] от $POSITION
\item[] $NAME
\end{itemize}

\begin{center}
  Заявление
\end{center}

Прошу предоставить мне очередной оплачиваемый отпуск с $SINCE по $TILL.

\bigskip

$(date +%d.%m.%Y)
\hfill{/${SIGN}/}
\end{document}
EOF

pdflatex --interaction batchmode  "vacation.tex" 2>&1 > /dev/null
popd >/dev/null

cp "$DNAME/vacation.pdf" "$FNAME.pdf"
